
from contextlib import ExitStack
from dateutil.relativedelta import relativedelta
from unittest.mock import patch
from odoo import fields, release
from odoo.addons.saas_trial.tests.common import TestTrialCommon, tagged
from odoo.addons.saas_worker import metabase, trials
from odoo.addons.saas_worker.util import version_gte, humanize_version
from odoo.addons.saas_worker.controllers import main


@tagged('post_install', '-at_install', 'post_saas_trial_creation')
class TestEdu(TestTrialCommon):

    def _patch_edu_prefix(self, prefix, edu_eol_dates=None):
        patch_stack = ExitStack()
        patch_stack.enter_context(patch.object(main, "EDU_PREFIX", new=self.env.cr.dbname))
        patch_stack.enter_context(patch.object(trials, "EDU_PREFIX", new=self.env.cr.dbname))
        if edu_eol_dates:
            patch_stack.enter_context(patch('odoo.addons.saas_worker.util.EDU_EOL_DATES', new=edu_eol_dates))
        return patch_stack

    def test_edu_end_of_life_warning(self):
        """Test that a warning is displayed for EOL edu versions."""
        metabase.set_expiry_in(
            self.env.cr.dbname, '2 month',
            set_status='activated', set_extra_apps=False,
        )
        test_eol_dates = {
            humanize_version(release.serie): \
                fields.Date.to_string(fields.Date.today() + relativedelta(days=32))
        }
        with self._patch_edu_prefix(self.env.cr.dbname, edu_eol_dates=test_eol_dates):
            self.start_tour('/web', 'trial_edu_eol_warning', login=self.user_admin.login)

    def test_edu_expiration_warning(self):
        """Test that a warning is displayed for edu at least 2 months before expiration date"""
        if not version_gte("15.0"):
            self.skipTest("edu expiration only for versions >= 15.0")

        # edu has a lifetime of 24 months (2 years) max, so set it's creation
        # date to 22 months in the past to avoid `bump_sliding_expiration`
        # moving it further than 2 months
        with metabase.cursor() as mcr:
            mcr.execute("""
                UPDATE databases
                   SET create_date = now() - '22 months'::interval
                 WHERE name = %s
            """, (self.env.cr.dbname,))
        metabase.set_expiry_in(
            self.env.cr.dbname, '2 month',
            set_status='activated', set_extra_apps=False,
        )

        with self._patch_edu_prefix(self.env.cr.dbname):
            self.start_tour('/web', 'trial_edu_expires_warning', login=self.user_admin.login)
